@@ -8,7 +8,7 @@ use IO::String;
 
 # ABSTRACT: ISO ANSI C 2011 grammar written in Marpa BNF
 
-our $VERSION = '0.39'; # VERSION
+our $VERSION = '0.42'; # VERSION
 
 
 our %DEFAULT_PAUSE = (
@@ -26,14 +26,20 @@ our %DEFAULT_PAUSE = (
     ANY_ASM              => 'after',
 );
 
+our $DEFAULTACTIONOBJECT = sprintf('%s::%s', __PACKAGE__, 'Actions');
+our $DEFAULTNONTERMINALSEMANTIC = ':default ::= action => [values] bless => ::lhs';
+our $DEFAULTTERMINALSEMANTIC = 'lexeme default = action => [start,length,value,name] forgiving => 1';
+
 our $DATA = do {local $/; <DATA>};
 
 sub new {
-  my ($class, $pausep, $start) = @_;
+  my ($class, $pausep, $start, $actionObject, $nonTerminalSemantic, $terminalSemantic) = @_;
+
+  $actionObject //= $DEFAULTACTIONOBJECT;
 
   my $self  = {
-    _grammar_option => {action_object  => sprintf('%s::%s', __PACKAGE__, 'Actions')},
-    _recce_option => {ranking_method => 'high_rule_only'},
+    _grammar_option => {},
+    _recce_option => {semantics_package => $actionObject, ranking_method => 'high_rule_only'},
   };
   #
   # Rework the grammar to have the pauses:
@@ -84,9 +90,13 @@ sub new {
       }
       $self->{_content} .= $line;
   }
+  $nonTerminalSemantic //= $DEFAULTNONTERMINALSEMANTIC;
+  $terminalSemantic //= $DEFAULTTERMINALSEMANTIC;
 
   $self->{_content} =~ s/\$PRAGMAS\n/$pragmas/;
   $self->{_content} =~ s/\$START\n/$start/;
+  $self->{_content} =~ s/\$NONTERMINALSEMANTIC\b/$nonTerminalSemantic/;
+  $self->{_content} =~ s/\$TERMINALSEMANTIC\b/$terminalSemantic/;
 
   bless($self, $class);
 
@@ -123,7 +133,7 @@ MarpaX::Languages::C::AST::Grammar::ISO_ANSI_C_2011 - ISO ANSI C 2011 grammar wr
 
 =head1 VERSION
 
-version 0.39
+version 0.42
 
 =head1 SYNOPSIS
 
@@ -143,9 +153,9 @@ This modules contains the ISO ANSI C 2011 C grammar written in Marpa BNF, as of
 
 =head1 SUBROUTINES/METHODS
 
-=head2 new([$pausep, $start])
+=head2 new([$pausep, $start, $actionObject, $nonTerminalSemantic, $terminalSemantic])
 
-Instance a new object. Takes an eventual reference to a HASH for lexemes for which a pause after is requested, followed by an eventual start rule. Default paused lexemes is hardcoded to a list of lexeme that must always be paused, and this list cannot be altered. Default start rule is 'translationUnit'.
+Instance a new object. Takes an eventual reference to a HASH for lexemes for which a pause after is requested, followed by an eventual start rule, an eventual action object, an eventual default non-terminal semantic action, a default terminal semantic action. Default paused lexemes is hardcoded to a list of lexeme that must always be paused, and this list cannot be altered. Default start rule is 'translationUnit'. Default action object is hardcoded to __PACKAGE__::Actions module. Default non-terminal semantic is hardcoded to ':default ::= action => [values] bless => ::lhs'. Default terminal semantic is hardcoded to :'lexeme default = action => [start,length,value] forgiving => 1'.
 
 =head2 content()
 
@@ -184,8 +194,8 @@ $PRAGMAS
 #
 # Defaults
 #
-:default ::= action => [values] bless => ::lhs
-lexeme default = action => [start,length,value] forgiving => 1
+$NONTERMINALSEMANTIC
+$TERMINALSEMANTIC
 
 #
 # G1 (grammar), c.f. http://www.quut.com/c/ANSI-C-grammar-y-2011.html
@@ -222,9 +232,7 @@ string
 genericSelection
 	::= GENERIC LPAREN assignmentExpression COMMA genericAssocList RPAREN
 
-genericAssocList
-	::= genericAssociation
-	| genericAssocList COMMA genericAssociation
+genericAssocList ::= genericAssociation+ separator => COMMA proper => 1
 
 genericAssociation
 	::= typeName COLON assignmentExpression
@@ -429,9 +437,7 @@ declarationSpecifiers2 ::=       # List with one or more typeSpecifier2
 
 # declarationSpecifiers ::= declarationSpecifiersUnit+
 
-initDeclaratorList
-	::= initDeclarator
-	| initDeclaratorList COMMA initDeclarator
+initDeclaratorList ::= initDeclarator+ separator => COMMA proper => 1
 
 initDeclarator
 	::= declarator EQUAL initializer
@@ -484,8 +490,8 @@ event 'structContextEnd[]' = nulled <structContextEnd>
 structContextEnd ::=
 
 structOrUnionSpecifier
-	::= structOrUnion LCURLY <structContextStart> structDeclarationList RCURLY <structContextEnd>
-	| structOrUnion IDENTIFIER_UNAMBIGUOUS LCURLY <structContextStart> structDeclarationList RCURLY <structContextEnd>
+	::= structOrUnion LCURLY (<structContextStart>) structDeclarationList RCURLY (<structContextEnd>)
+	| structOrUnion IDENTIFIER_UNAMBIGUOUS LCURLY (<structContextStart>) structDeclarationList RCURLY (<structContextEnd>)
 	| structOrUnion IDENTIFIER_UNAMBIGUOUS
 
 structOrUnion
@@ -528,9 +534,7 @@ specifierQualifierList2 ::= # List with one or more typeSpecifier2
                           | specifierQualifierList2 typeQualifier
                           | specifierQualifierList2 gccExtension
 
-structDeclaratorList
-	::= structDeclarator
-	| structDeclaratorList COMMA structDeclarator
+structDeclaratorList ::= structDeclarator+ separator => COMMA proper => 1
 
 structDeclarator
 	::= COLON constantExpression
@@ -539,14 +543,13 @@ structDeclarator
 
 enumSpecifier
 	::= ENUM LCURLY enumeratorList RCURLY
-	| ENUM LCURLY enumeratorList COMMA RCURLY
 	| ENUM IDENTIFIER_UNAMBIGUOUS LCURLY enumeratorList RCURLY
-	| ENUM IDENTIFIER_UNAMBIGUOUS LCURLY enumeratorList COMMA RCURLY
 	| ENUM IDENTIFIER_UNAMBIGUOUS
 
-enumeratorList
-	::= enumerator
-	| enumeratorList COMMA enumerator
+#
+# Saying 0 allow to have a final COMMA after the list
+#
+enumeratorList ::= enumerator+ separator => COMMA proper => 0
 
 enumerator	# identifiers must be flagged as ENUMERATION_CONSTANT
 	::= enumerationConstant EQUAL constantExpression
@@ -597,12 +600,12 @@ directDeclarator
 	| LPAREN declarator RPAREN
 	| directDeclarator LBRACKET RBRACKET
 	| directDeclarator LBRACKET STAR RBRACKET
-	| directDeclarator LBRACKET STATIC gccArrayTypeModifierList assignmentExpression RBRACKET
+	| directDeclarator LBRACKET STATIC typeQualifierList assignmentExpression RBRACKET
 	| directDeclarator LBRACKET STATIC assignmentExpression RBRACKET
-	| directDeclarator LBRACKET gccArrayTypeModifierList STAR RBRACKET
-	| directDeclarator LBRACKET gccArrayTypeModifierList STATIC assignmentExpression RBRACKET
-	| directDeclarator LBRACKET gccArrayTypeModifierList assignmentExpression RBRACKET
-	| directDeclarator LBRACKET gccArrayTypeModifierList RBRACKET
+	| directDeclarator LBRACKET typeQualifierList STAR RBRACKET
+	| directDeclarator LBRACKET typeQualifierList STATIC assignmentExpression RBRACKET
+	| directDeclarator LBRACKET typeQualifierList assignmentExpression RBRACKET
+	| directDeclarator LBRACKET typeQualifierList RBRACKET
 	| directDeclarator LBRACKET assignmentExpression RBRACKET
 	| directDeclarator LPAREN_SCOPE parameterTypeList RPAREN_SCOPE
 	| directDeclarator LPAREN_SCOPE RPAREN_SCOPE
@@ -618,7 +621,7 @@ pointer
 	| msvsAttributeAny STAR pointer
 	| msvsAttributeAny STAR
 
-gccArrayTypeModifierList ::= gccArrayTypeModifier+
+typeQualifierList ::= typeQualifier+
 
 #typeQualifierList
 #	::= typeQualifier
@@ -628,9 +631,7 @@ parameterTypeList
 	::= parameterList COMMA ELLIPSIS
 	| parameterList
 
-parameterList
-	::= parameterDeclaration
-	| parameterList COMMA parameterDeclaration
+parameterList ::= parameterDeclaration+ separator => COMMA proper => 1
 
 event 'parameterDeclarationdeclarationSpecifiers$' = completed <parameterDeclarationdeclarationSpecifiers>
 parameterDeclarationdeclarationSpecifiers ::= declarationSpecifiers
@@ -646,9 +647,7 @@ parameterDeclaration
 	| declarationSpecifiers abstractDeclarator  rank => -1
 	| declarationSpecifiers                     rank => -2
 
-identifierList
-	::= IDENTIFIER
-	| identifierList COMMA IDENTIFIER
+identifierList ::= IDENTIFIER+ separator => COMMA proper => 1
 
 typeName
 	::= specifierQualifierList abstractDeclarator
@@ -665,19 +664,19 @@ directAbstractDeclarator
 	::= LPAREN abstractDeclarator RPAREN
 	| LBRACKET RBRACKET
 	| LBRACKET STAR RBRACKET
-	| LBRACKET STATIC gccArrayTypeModifierList assignmentExpression RBRACKET
+	| LBRACKET STATIC typeQualifierList assignmentExpression RBRACKET
 	| LBRACKET STATIC assignmentExpression RBRACKET
-	| LBRACKET gccArrayTypeModifierList STATIC assignmentExpression RBRACKET
-	| LBRACKET gccArrayTypeModifierList assignmentExpression RBRACKET
-	| LBRACKET gccArrayTypeModifierList RBRACKET
+	| LBRACKET typeQualifierList STATIC assignmentExpression RBRACKET
+	| LBRACKET typeQualifierList assignmentExpression RBRACKET
+	| LBRACKET typeQualifierList RBRACKET
 	| LBRACKET assignmentExpression RBRACKET
 	| directAbstractDeclarator LBRACKET RBRACKET
 	| directAbstractDeclarator LBRACKET STAR RBRACKET
-	| directAbstractDeclarator LBRACKET STATIC gccArrayTypeModifierList assignmentExpression RBRACKET
+	| directAbstractDeclarator LBRACKET STATIC typeQualifierList assignmentExpression RBRACKET
 	| directAbstractDeclarator LBRACKET STATIC assignmentExpression RBRACKET
-	| directAbstractDeclarator LBRACKET gccArrayTypeModifierList assignmentExpression RBRACKET
-	| directAbstractDeclarator LBRACKET gccArrayTypeModifierList STATIC assignmentExpression RBRACKET
-	| directAbstractDeclarator LBRACKET gccArrayTypeModifierList RBRACKET
+	| directAbstractDeclarator LBRACKET typeQualifierList assignmentExpression RBRACKET
+	| directAbstractDeclarator LBRACKET typeQualifierList STATIC assignmentExpression RBRACKET
+	| directAbstractDeclarator LBRACKET typeQualifierList RBRACKET
 	| directAbstractDeclarator LBRACKET assignmentExpression RBRACKET
 	| LPAREN_SCOPE RPAREN_SCOPE
 	| LPAREN_SCOPE parameterTypeList RPAREN_SCOPE
@@ -721,7 +720,8 @@ statement
 
 labeledStatement
 	::= IDENTIFIER COLON statement
-	| CASE constantExpression COLON statement
+	| CASE constantExpression           COLON statement
+	| CASE constantExpression (WS_MANY) COLON statement
 	| CASE constantExpression (WS_MANY) ELLIPSIS (WS_MANY) constantExpression COLON statement
 	| DEFAULT COLON statement
 
@@ -766,7 +766,7 @@ event '^externalDeclaration' = predicted <externalDeclaration>
 externalDeclaration
 	::= functionDefinition
 	| declaration
-        | (SEMICOLON)
+        | SEMICOLON
 
 compoundStatementReenterScope ::= LCURLY_REENTERSCOPE RCURLY_SCOPE
 	                        | LCURLY_REENTERSCOPE blockItemList RCURLY_SCOPE
@@ -1327,8 +1327,6 @@ gccAsmClobber ::= string
 
 gccStatementExpression ::= LPAREN compoundStatement RPAREN
 
-gccArrayTypeModifier ::= typeQualifier
-
 # @since 2.6.264
 # for error handling: second assignmentExpression is always last parameter name
 gccBuiltinVaStart ::= GCC_BUILTIN_VA_START
